@@ -1,10 +1,28 @@
-from __future__ import absolute_import, unicode_literals
-
-import six
-
-from .types import Archive, Boundary, Broadcast, Campaign, CampaignEvent, Channel, ChannelEvent, Contact, Export, Field
-from .types import FlowStart, Flow, Group, Label, Message, Org, Resthook, ResthookSubscriber, ResthookEvent, Run
-from ..clients import BaseCursorClient
+from ..base import BaseCursorClient
+from .types import (
+    Archive,
+    Boundary,
+    Broadcast,
+    Campaign,
+    CampaignEvent,
+    Channel,
+    ChannelEvent,
+    Classifier,
+    Contact,
+    Export,
+    Field,
+    Flow,
+    FlowStart,
+    Global,
+    Group,
+    Label,
+    Message,
+    Org,
+    Resthook,
+    ResthookEvent,
+    ResthookSubscriber,
+    Run,
+)
 
 
 class TembaClient(BaseCursorClient):
@@ -15,6 +33,7 @@ class TembaClient(BaseCursorClient):
     :param str token: organization API token
     :param str user_agent: string to be included in the User-Agent header
     """
+
     def __init__(self, host, token, user_agent=None, verify_ssl=None):
         super(TembaClient, self).__init__(host, token, 2, user_agent, verify_ssl)
 
@@ -34,7 +53,7 @@ class TembaClient(BaseCursorClient):
         """
 
         params = self._build_params(archive_type=archive_type, period=period, before=before, after=after)
-        return self._get_query('archives', params, Archive)
+        return self._get_query("archives", params, Archive)
 
     def get_boundaries(self, geometry=None):
         """
@@ -43,7 +62,7 @@ class TembaClient(BaseCursorClient):
         :return: boundary query
         """
         params = self._build_params(geometry=geometry)
-        return self._get_query('boundaries', params, Boundary)
+        return self._get_query("boundaries", params, Boundary)
 
     def get_broadcasts(self, id=None, before=None, after=None):
         """
@@ -55,7 +74,7 @@ class TembaClient(BaseCursorClient):
         :return: broadcast query
         """
         params = self._build_params(id=id, before=before, after=after)
-        return self._get_query('broadcasts', params, Broadcast)
+        return self._get_query("broadcasts", params, Broadcast)
 
     def get_campaigns(self, uuid=None):
         """
@@ -65,7 +84,7 @@ class TembaClient(BaseCursorClient):
         :return: campaign query
         """
         params = self._build_params(uuid=uuid)
-        return self._get_query('campaigns', params, Campaign)
+        return self._get_query("campaigns", params, Campaign)
 
     def get_campaign_events(self, uuid=None, campaign=None):
         """
@@ -76,7 +95,7 @@ class TembaClient(BaseCursorClient):
         :return: campaign event query
         """
         params = self._build_params(uuid=uuid, campaign=campaign)
-        return self._get_query('campaign_events', params, CampaignEvent)
+        return self._get_query("campaign_events", params, CampaignEvent)
 
     def get_channels(self, uuid=None, address=None):
         """
@@ -87,7 +106,7 @@ class TembaClient(BaseCursorClient):
         :return: channel query
         """
         params = self._build_params(uuid=uuid, address=address)
-        return self._get_query('channels', params, Channel)
+        return self._get_query("channels", params, Channel)
 
     def get_channel_events(self, id=None, contact=None, before=None, after=None):
         """
@@ -100,7 +119,17 @@ class TembaClient(BaseCursorClient):
         :return: channel event query
         """
         params = self._build_params(id=id, contact=contact, before=before, after=after)
-        return self._get_query('channel_events', params, ChannelEvent)
+        return self._get_query("channel_events", params, ChannelEvent)
+
+    def get_classifiers(self, uuid=None):
+        """
+        Gets all matching NLU classifiers
+
+        :param uuid: classifier UUID
+        :return: classifier query
+        """
+        params = self._build_params(uuid=uuid)
+        return self._get_query("classifiers", params, Classifier)
 
     def get_contacts(self, uuid=None, urn=None, group=None, deleted=None, before=None, after=None):
         """
@@ -115,7 +144,7 @@ class TembaClient(BaseCursorClient):
         :return: contact query
         """
         params = self._build_params(uuid=uuid, urn=urn, group=group, deleted=deleted, before=before, after=after)
-        return self._get_query('contacts', params, Contact)
+        return self._get_query("contacts", params, Contact)
 
     def get_definitions(self, flows=(), campaigns=(), dependencies=None):
         """
@@ -127,7 +156,7 @@ class TembaClient(BaseCursorClient):
         :return: definitions export
         """
         params = self._build_params(flow=flows, campaign=campaigns, dependencies=dependencies)
-        return Export.deserialize(self._get_raw('definitions', params))
+        return Export.deserialize(self._get_raw("definitions", params))
 
     def get_fields(self, key=None):
         """
@@ -136,7 +165,7 @@ class TembaClient(BaseCursorClient):
         :param key: field key
         :return: field query
         """
-        return self._get_query('fields', self._build_params(key=key), Field)
+        return self._get_query("fields", self._build_params(key=key), Field)
 
     def get_flows(self, uuid=None):
         """
@@ -145,7 +174,7 @@ class TembaClient(BaseCursorClient):
         :param uuid: flow UUID
         :return: flow query
         """
-        return self._get_query('flows', self._build_params(uuid=uuid), Flow)
+        return self._get_query("flows", self._build_params(uuid=uuid), Flow)
 
     def get_flow_starts(self, uuid=None):
         """
@@ -154,7 +183,15 @@ class TembaClient(BaseCursorClient):
         :param uuid: flow start UUID
         :return: flow start query
         """
-        return self._get_query('flow_starts', self._build_params(uuid=uuid), FlowStart)
+        return self._get_query("flow_starts", self._build_params(uuid=uuid), FlowStart)
+
+    def get_globals(self):
+        """
+        Gets all globals
+
+        :return: global query
+        """
+        return self._get_query("globals", {}, Global)
 
     def get_groups(self, uuid=None, name=None):
         """
@@ -164,7 +201,7 @@ class TembaClient(BaseCursorClient):
         :param name: group name
         :return: group query
         """
-        return self._get_query('groups', self._build_params(uuid=uuid, name=name), Group)
+        return self._get_query("groups", self._build_params(uuid=uuid, name=name), Group)
 
     def get_labels(self, uuid=None, name=None):
         """
@@ -174,7 +211,7 @@ class TembaClient(BaseCursorClient):
         :param name: label name
         :return: label query
         """
-        return self._get_query('labels', self._build_params(uuid=uuid, name=name), Label)
+        return self._get_query("labels", self._build_params(uuid=uuid, name=name), Label)
 
     def get_messages(self, id=None, broadcast=None, contact=None, folder=None, label=None, before=None, after=None):
         """
@@ -189,9 +226,10 @@ class TembaClient(BaseCursorClient):
         :param datetime after: created after
         :return: message query
         """
-        params = self._build_params(id=id, broadcast=broadcast, contact=contact, folder=folder, label=label,
-                                    before=before, after=after)
-        return self._get_query('messages', params, Message)
+        params = self._build_params(
+            id=id, broadcast=broadcast, contact=contact, folder=folder, label=label, before=before, after=after
+        )
+        return self._get_query("messages", params, Message)
 
     def get_org(self, retry_on_rate_exceed=False):
         """
@@ -200,7 +238,7 @@ class TembaClient(BaseCursorClient):
         :param retry_on_rate_exceed: whether to sleep and retry if request rate limit exceeded
         :return: the org
         """
-        return Org.deserialize(self._get_raw('org', {}, retry_on_rate_exceed))
+        return Org.deserialize(self._get_raw("org", {}, retry_on_rate_exceed))
 
     def get_resthooks(self):
         """
@@ -208,7 +246,7 @@ class TembaClient(BaseCursorClient):
 
         :return: resthook query
         """
-        return self._get_query('resthooks', {}, Resthook)
+        return self._get_query("resthooks", {}, Resthook)
 
     def get_resthook_events(self, resthook=None):
         """
@@ -218,7 +256,7 @@ class TembaClient(BaseCursorClient):
         :return: resthook event query
         """
         params = self._build_params(resthook=resthook)
-        return self._get_query('resthook_events', params, ResthookEvent)
+        return self._get_query("resthook_events", params, ResthookEvent)
 
     def get_resthook_subscribers(self, id=None, resthook=None):
         """
@@ -229,7 +267,7 @@ class TembaClient(BaseCursorClient):
         :return: resthook subscriber query
         """
         params = self._build_params(id=id, resthook=resthook)
-        return self._get_query('resthook_subscribers', params, ResthookSubscriber)
+        return self._get_query("resthook_subscribers", params, ResthookSubscriber)
 
     def get_runs(self, id=None, flow=None, contact=None, responded=None, before=None, after=None):
         """
@@ -244,7 +282,7 @@ class TembaClient(BaseCursorClient):
         :return: flow run query
         """
         params = self._build_params(id=id, flow=flow, contact=contact, responded=responded, before=before, after=after)
-        return self._get_query('runs', params, Run)
+        return self._get_query("runs", params, Run)
 
     # ==================================================================================================================
     # Create object operations
@@ -261,7 +299,7 @@ class TembaClient(BaseCursorClient):
         :return: the new broadcast
         """
         payload = self._build_params(text=text, urns=urns, contacts=contacts, groups=groups)
-        return Broadcast.deserialize(self._post('broadcasts', None, payload))
+        return Broadcast.deserialize(self._post("broadcasts", None, payload))
 
     def create_campaign(self, name, group):
         """
@@ -272,7 +310,7 @@ class TembaClient(BaseCursorClient):
         :return: the new campaign
         """
         payload = self._build_params(name=name, group=group)
-        return Campaign.deserialize(self._post('campaigns', None, payload))
+        return Campaign.deserialize(self._post("campaigns", None, payload))
 
     def create_campaign_event(self, campaign, relative_to, offset, unit, delivery_hour, message=None, flow=None):
         """
@@ -287,9 +325,16 @@ class TembaClient(BaseCursorClient):
         :param * flow: flow object, UUID or name
         :return: the new campaign
         """
-        payload = self._build_params(campaign=campaign, relative_to=relative_to, offset=offset, unit=unit,
-                                     delivery_hour=delivery_hour, message=message, flow=flow)
-        return CampaignEvent.deserialize(self._post('campaign_events', None, payload))
+        payload = self._build_params(
+            campaign=campaign,
+            relative_to=relative_to,
+            offset=offset,
+            unit=unit,
+            delivery_hour=delivery_hour,
+            message=message,
+            flow=flow,
+        )
+        return CampaignEvent.deserialize(self._post("campaign_events", None, payload))
 
     def create_contact(self, name=None, language=None, urns=None, fields=None, groups=None):
         """
@@ -303,7 +348,7 @@ class TembaClient(BaseCursorClient):
         :return: the new contact
         """
         payload = self._build_params(name=name, language=language, urns=urns, fields=fields, groups=groups)
-        return Contact.deserialize(self._post('contacts', None, payload))
+        return Contact.deserialize(self._post("contacts", None, payload))
 
     def create_field(self, label, value_type):
         """
@@ -313,9 +358,9 @@ class TembaClient(BaseCursorClient):
         :param str value_type: field value type
         :return: the new field
         """
-        return Field.deserialize(self._post('fields', None, self._build_params(label=label, value_type=value_type)))
+        return Field.deserialize(self._post("fields", None, self._build_params(label=label, value_type=value_type)))
 
-    def create_flow_start(self, flow, urns=None, contacts=None, groups=None, restart_participants=None, extra=None):
+    def create_flow_start(self, flow, urns=None, contacts=None, groups=None, restart_participants=None, params=None):
         """
         Creates a new flow start
 
@@ -324,12 +369,29 @@ class TembaClient(BaseCursorClient):
         :param list[str] contacts: UUIDs of contacts to start
         :param list[str] groups: UUIDs of contact groups to start
         :param bool restart_participants: whether to restart participants already in this flow
-        :param * extra: a dictionary of extra parameters to pass to the flow
+        :param * params: a dictionary of parameters to pass to the flow
         :return: the new label
         """
-        payload = self._build_params(flow=flow, urns=urns, contacts=contacts, groups=groups,
-                                     restart_participants=restart_participants, extra=extra)
-        return FlowStart.deserialize(self._post('flow_starts', None, payload))
+        payload = self._build_params(
+            flow=flow,
+            urns=urns,
+            contacts=contacts,
+            groups=groups,
+            restart_participants=restart_participants,
+            params=params,
+        )
+        return FlowStart.deserialize(self._post("flow_starts", None, payload))
+
+    def create_global(self, name, value):
+        """
+        Creates a new global
+
+        :param str name: global name
+        :param str value: global value
+        :return: the new global
+        """
+        payload = self._build_params(name=name, value=value)
+        return Global.deserialize(self._post("globals", None, payload))
 
     def create_group(self, name):
         """
@@ -338,7 +400,7 @@ class TembaClient(BaseCursorClient):
         :param str name: group name
         :return: the new group
         """
-        return Group.deserialize(self._post('groups', None, self._build_params(name=name)))
+        return Group.deserialize(self._post("groups", None, self._build_params(name=name)))
 
     def create_label(self, name):
         """
@@ -347,7 +409,7 @@ class TembaClient(BaseCursorClient):
         :param str name: label name
         :return: the new label
         """
-        return Label.deserialize(self._post('labels', None, self._build_params(name=name)))
+        return Label.deserialize(self._post("labels", None, self._build_params(name=name)))
 
     def create_resthook_subscriber(self, resthook, target_url):
         """
@@ -358,7 +420,7 @@ class TembaClient(BaseCursorClient):
         :return: the new subscriber
         """
         payload = self._build_params(resthook=resthook, target_url=target_url)
-        return ResthookSubscriber.deserialize(self._post('resthook_subscribers', None, payload))
+        return ResthookSubscriber.deserialize(self._post("resthook_subscribers", None, payload))
 
     # ==================================================================================================================
     # Update object operations
@@ -375,7 +437,7 @@ class TembaClient(BaseCursorClient):
         """
         params = self._build_id_param(uuid=campaign)
         payload = self._build_params(name=name, group=group)
-        return Campaign.deserialize(self._post('campaigns', params, payload))
+        return Campaign.deserialize(self._post("campaigns", params, payload))
 
     def update_campaign_event(self, event, relative_to, offset, unit, delivery_hour, message=None, flow=None):
         """
@@ -391,9 +453,10 @@ class TembaClient(BaseCursorClient):
         :return: the new campaign
         """
         params = self._build_id_param(uuid=event)
-        payload = self._build_params(relative_to=relative_to, offset=offset, unit=unit,
-                                     delivery_hour=delivery_hour, message=message, flow=flow)
-        return CampaignEvent.deserialize(self._post('campaign_events', params, payload))
+        payload = self._build_params(
+            relative_to=relative_to, offset=offset, unit=unit, delivery_hour=delivery_hour, message=message, flow=flow
+        )
+        return CampaignEvent.deserialize(self._post("campaign_events", params, payload))
 
     def update_contact(self, contact, name=None, language=None, urns=None, fields=None, groups=None):
         """
@@ -407,10 +470,10 @@ class TembaClient(BaseCursorClient):
         :param list groups: list of group objects or UUIDs
         :return: the updated contact
         """
-        is_urn = isinstance(contact, six.string_types) and ':' in contact
-        params = self._build_id_param(**{'urn' if is_urn else 'uuid': contact})
+        is_urn = isinstance(contact, str) and ":" in contact
+        params = self._build_id_param(**{"urn" if is_urn else "uuid": contact})
         payload = self._build_params(name=name, language=language, urns=urns, fields=fields, groups=groups)
-        return Contact.deserialize(self._post('contacts', params, self._build_params(**payload)))
+        return Contact.deserialize(self._post("contacts", params, self._build_params(**payload)))
 
     def update_field(self, field, label, value_type):
         """
@@ -423,7 +486,19 @@ class TembaClient(BaseCursorClient):
         """
         params = self._build_id_param(key=field)
         payload = self._build_params(label=label, value_type=value_type)
-        return Field.deserialize(self._post('fields', params, payload))
+        return Field.deserialize(self._post("fields", params, payload))
+
+    def update_global(self, glbl, value):
+        """
+        Updates an existing global
+
+        :param * glbl: global object or key
+        :param str value: new value
+        :return: the updated global
+        """
+        params = self._build_id_param(key=glbl)
+        payload = self._build_params(value=value)
+        return Global.deserialize(self._post("globals", params, payload))
 
     def update_group(self, group, name):
         """
@@ -433,7 +508,7 @@ class TembaClient(BaseCursorClient):
         :param str name: group name
         :return: the updated group
         """
-        return Group.deserialize(self._post('groups', self._build_id_param(uuid=group), self._build_params(name=name)))
+        return Group.deserialize(self._post("groups", self._build_id_param(uuid=group), self._build_params(name=name)))
 
     def update_label(self, label, name):
         """
@@ -443,7 +518,7 @@ class TembaClient(BaseCursorClient):
         :param str name: label name
         :return: the updated label
         """
-        return Label.deserialize(self._post('labels', self._build_id_param(uuid=label), self._build_params(name=name)))
+        return Label.deserialize(self._post("labels", self._build_id_param(uuid=label), self._build_params(name=name)))
 
     # ==================================================================================================================
     # Delete object operations
@@ -455,7 +530,7 @@ class TembaClient(BaseCursorClient):
 
         :param * event: campaign event object or UUID
         """
-        self._delete('campaign_events', self._build_id_param(uuid=event))
+        self._delete("campaign_events", self._build_id_param(uuid=event))
 
     def delete_contact(self, contact):
         """
@@ -463,9 +538,9 @@ class TembaClient(BaseCursorClient):
 
         :param * contact: contact object, UUID or URN
         """
-        is_urn = isinstance(contact, six.string_types) and ':' in contact
-        params = self._build_id_param(**{'urn' if is_urn else 'uuid': contact})
-        self._delete('contacts', params)
+        is_urn = isinstance(contact, str) and ":" in contact
+        params = self._build_id_param(**{"urn" if is_urn else "uuid": contact})
+        self._delete("contacts", params)
 
     def delete_group(self, group):
         """
@@ -473,7 +548,7 @@ class TembaClient(BaseCursorClient):
 
         :param * group: group object or UUID
         """
-        self._delete('groups', self._build_id_param(uuid=group))
+        self._delete("groups", self._build_id_param(uuid=group))
 
     def delete_label(self, label):
         """
@@ -481,7 +556,7 @@ class TembaClient(BaseCursorClient):
 
         :param * label: label object or UUID
         """
-        self._delete('labels', self._build_id_param(uuid=label))
+        self._delete("labels", self._build_id_param(uuid=label))
 
     def delete_resthook_subscriber(self, subscriber):
         """
@@ -489,7 +564,7 @@ class TembaClient(BaseCursorClient):
 
         :param * subscriber: the resthook subscriber or id
         """
-        self._delete('resthook_subscribers', self._build_id_param(id=subscriber))
+        self._delete("resthook_subscribers", self._build_id_param(id=subscriber))
 
     # ==================================================================================================================
     # Bulk object operations
@@ -502,7 +577,7 @@ class TembaClient(BaseCursorClient):
         :param list[*] contacts: contact objects, UUIDs or URNs
         :param * group: contact group object or UUID
         """
-        self._post('contact_actions', None, self._build_params(contacts=contacts, action='add', group=group))
+        self._post("contact_actions", None, self._build_params(contacts=contacts, action="add", group=group))
 
     def bulk_remove_contacts(self, contacts, group):
         """
@@ -511,7 +586,7 @@ class TembaClient(BaseCursorClient):
         :param list[*] contacts: contact objects, UUIDs or URNs
         :param * group: contact group object or UUID
         """
-        self._post('contact_actions', None, self._build_params(contacts=contacts, action='remove', group=group))
+        self._post("contact_actions", None, self._build_params(contacts=contacts, action="remove", group=group))
 
     def bulk_block_contacts(self, contacts):
         """
@@ -519,7 +594,7 @@ class TembaClient(BaseCursorClient):
 
         :param list[*] contacts: contact objects, UUIDs or URNs
         """
-        self._post('contact_actions', None, self._build_params(contacts=contacts, action='block'))
+        self._post("contact_actions", None, self._build_params(contacts=contacts, action="block"))
 
     def bulk_unblock_contacts(self, contacts):
         """
@@ -527,7 +602,7 @@ class TembaClient(BaseCursorClient):
 
         :param list[*] contacts: contact objects, UUIDs or URNs
         """
-        self._post('contact_actions', None, self._build_params(contacts=contacts, action='unblock'))
+        self._post("contact_actions", None, self._build_params(contacts=contacts, action="unblock"))
 
     def bulk_interrupt_contacts(self, contacts):
         """
@@ -535,15 +610,15 @@ class TembaClient(BaseCursorClient):
 
         :param list[*] contacts: contact objects, UUIDs or URNs
         """
-        self._post('contact_actions', None, self._build_params(contacts=contacts, action='interrupt'))
+        self._post("contact_actions", None, self._build_params(contacts=contacts, action="interrupt"))
 
-    def bulk_archive_contacts(self, contacts):
+    def bulk_archive_contact_messages(self, contacts):
         """
         Archives all messages for contacts
 
         :param list[*] contacts: contact objects, UUIDs or URNs
         """
-        self._post('contact_actions', None, self._build_params(contacts=contacts, action='archive'))
+        self._post("contact_actions", None, self._build_params(contacts=contacts, action="archive_messages"))
 
     def bulk_delete_contacts(self, contacts):
         """
@@ -551,7 +626,7 @@ class TembaClient(BaseCursorClient):
 
         :param list[*] contacts: contact objects, UUIDs or URNs
         """
-        self._post('contact_actions', None, self._build_params(contacts=contacts, action='delete'))
+        self._post("contact_actions", None, self._build_params(contacts=contacts, action="delete"))
 
     def bulk_label_messages(self, messages, label=None, label_name=None):
         """
@@ -561,8 +636,8 @@ class TembaClient(BaseCursorClient):
         :param * label: existing label object or UUID
         :param str label_name: label name which can be created if required
         """
-        payload = self._build_params(messages=messages, action='label', label=label, label_name=label_name)
-        self._post('message_actions', None, payload)
+        payload = self._build_params(messages=messages, action="label", label=label, label_name=label_name)
+        self._post("message_actions", None, payload)
 
     def bulk_unlabel_messages(self, messages, label=None, label_name=None):
         """
@@ -572,8 +647,8 @@ class TembaClient(BaseCursorClient):
         :param * label: existing label object or UUID
         :param str label_name: label name which is ignored if doesn't exist
         """
-        payload = self._build_params(messages=messages, action='unlabel', label=label, label_name=label_name)
-        self._post('message_actions', None, payload)
+        payload = self._build_params(messages=messages, action="unlabel", label=label, label_name=label_name)
+        self._post("message_actions", None, payload)
 
     def bulk_archive_messages(self, messages):
         """
@@ -581,7 +656,7 @@ class TembaClient(BaseCursorClient):
 
         :param list[*] messages: message objects or ids
         """
-        self._post('message_actions', None, self._build_params(messages=messages, action='archive'))
+        self._post("message_actions", None, self._build_params(messages=messages, action="archive"))
 
     def bulk_restore_messages(self, messages):
         """
@@ -589,7 +664,7 @@ class TembaClient(BaseCursorClient):
 
         :param list[*] messages: message objects or ids
         """
-        self._post('message_actions', None, self._build_params(messages=messages, action='restore'))
+        self._post("message_actions", None, self._build_params(messages=messages, action="restore"))
 
     def bulk_delete_messages(self, messages):
         """
@@ -597,4 +672,4 @@ class TembaClient(BaseCursorClient):
 
         :param list[*] messages: message objects or ids
         """
-        self._post('message_actions', None, self._build_params(messages=messages, action='delete'))
+        self._post("message_actions", None, self._build_params(messages=messages, action="delete"))
